import React, { Component } from 'react';
import PropTypes from 'prop-types';

import '../../style/generic/Modal.scss';

export default class Modal extends Component {
	static propTypes = {
		show: PropTypes.bool.isRequired,
		onClose: PropTypes.func.isRequired
	}

	componentDidMount(){
		if(this.props.show){
			this.hideBodyOverflow();
		}
	}

	componentDidUpdate(prevProps, prevState, snapshot){
		if(this.props.show){
			this.hideBodyOverflow();
		}else{
			this.resetBodyOverflow();
		}
	}
	
	componentWillUnmount(){
		this.resetBodyOverflow();
	}

	hideBodyOverflow(){
		document.body.style.overflow="hidden";
	}

	resetBodyOverflow(){
		document.body.style.overflow="auto";
	}

	render() {
		return (
			<div className="modal" style={{display: this.props.show ? "grid":"none"}}>
				<div className="modalForeground">
					<div className="modalTitle">{this.props.title}</div>
					<div className="modalClose icon-cancel-circled" onClick={this.props.onClose}></div>
					<div className="modalContent">{ this.props.children }</div>
				</div>
			</div>
		)
	}
}
