import React, { Component } from 'react'
import PropTypes from 'prop-types'

import { connect } from 'react-redux'; 
import { addTask, updateTask, deleteTask } from '../../action/TaskAction';

import '../../style/task/TaskForm.scss';
import DisplayError from '../generic/DisplayError';

class TaskForm extends Component {
	static propTypes = {
		task: PropTypes.object,
		addTask: PropTypes.func.isRequired,
		updateTask: PropTypes.func.isRequired,
		deleteTask: PropTypes.func.isRequired,
	}

	state={
		submitValue:"Create",
		isUpdate:false,
		task:{
			name: "",
			description: "",
			parent: ""
		},
		backendError:false
	}

	componentDidMount(){
		if(this.props.task){
			this.setState({
				"isUpdate":true,
				"submitValue":"Update",
				"task": {...this.state.task,...this.props.task}
			});
		}
		this.onBackendError=this.onBackendError.bind(this);
	}

	componentDidUpdate(prevProps, prevState, snapshot){
		if(this.props.task && this.props.task!==prevProps.task){
			this.setState({
				"isUpdate":true,
				"submitValue":"Update",
				"task": {...this.state.task,...this.props.task}
			});
		}
	}

	onChange=(e)=>{
		e.preventDefault();
		let n=e.target.name;
		let v=e.target.value;
		this.setState(prevState=>({
			task: {
				...prevState.task,
				[n]:v
			}
		}));
	}

	onSubmit=(e)=>{
		e.preventDefault();
		if(this.state.isUpdate){
			this.props.updateTask(this.state.task,this.onBackendError);
		}else{
			this.props.addTask(this.state.task,this.onBackendError);
		}
	}
	
	onBackendError(reason){
		this.setState({backendError:reason});
	}

	onDelete=(e)=>{
		e.preventDefault();
		this.props.deleteTask(this.state.task._id);
	}

	renderDeleteButton(){
		if(!this.state.isUpdate) return;
		return(
			<button className="taskFormDelete" onClick={this.onDelete}>Delete</button>
		);
	}
	
	renderFormElement(id,element,labelText=""){
		if(labelText==="") labelText=id;
		return(
			<React.Fragment>
				<label htmlFor={id}>{labelText}</label>
				{element}
			</React.Fragment>
		)
	}

	render() {
		let formName="";
		if(this.props.hasOwnProperty("name")) formName=this.props.name;
		return (
			<div className="taskForm">
				<form onSubmit={this.onSubmit}>
					{this.renderFormElement(
						"name"+formName,
						<input id={"name"+formName} type="text" name="name" placeholder="name" value={this.state.task.name} onChange={this.onChange}/>,
						"Name"
					)}
					{this.renderFormElement(
						"description"+formName,
						<textarea id={"description"+formName} type="text" name="description" placeholder="description" value={this.state.task.description} onChange={this.onChange}/>,
						"Description"
					)}
					{this.renderFormElement(
						"parent"+formName,
						<select id={"parent"+formName} name="parent" value={this.state.task.parent} onChange={this.onChange}>
							<option value="">No parent task</option>
							{this.props.tasks.map(task=>{
								return this.state.isUpdate && task._id === this.state.task._id ? "":<option key={task._id} value={task._id}>{task.name}</option>
							})}
						</select>,
						"Parent task"
					)}
					<input type="hidden" name="isUpdate" value={this.state.isUpdate}/>
					<input type="submit" value={this.state.submitValue}/>
				</form>
				<div>{this.state.backendError?<DisplayError axios={this.state.backendError}/>:""}</div>
				{this.renderDeleteButton()}
			</div>
		)
	}
}

const mapStateToProps = (state) => ({
	tasks: state.task.tasks
});

export default connect(mapStateToProps, {addTask, updateTask, deleteTask})(TaskForm);