import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { getTasks } from '../../action/TaskAction';
import Task from './Task';
import DisplayError from '../generic/DisplayError';
import '../../style/task/TaskList.scss';

class TaskList extends Component {
	static propTypes = {
		getTasks: PropTypes.func.isRequired,
		task: PropTypes.object.isRequired
	}
	state={
		error:false
	}
	componentDidMount(){
		this.props.getTasks((reason)=>{this.setState({"error":reason})});
	}
	renderTaskGroup(parent,generation=0){
		let taskCollection;
		if(generation===0){
			taskCollection=this.props.sortedTask.filter(task=>!task.parent)
		}else{
			taskCollection=this.props.sortedTask.filter(task=>task.parent===parent._id);
		}
		return(
			taskCollection.map(t=>{
				return(
					<div key={t._id} className={"taskGroup taskGroup_"+(generation)}>
						<Task task={t} />
						{this.renderTaskGroup(t,generation+1)}
					</div>
				)
			})
		)
	}
	render(){
		return (
			<React.Fragment>
				{(this.state && this.state.error)?<DisplayError axios={this.state.error}/>:""}
				<div className="taskList">{ this.renderTaskGroup() }</div>
			</React.Fragment>
		);
	}
}
const mapStateToProps = (state) => ({
	task: state.task,
	sortedTask: state.task.tasks.sort((a,b)=>(a.name).localeCompare(b.name))
});

export default connect(mapStateToProps, {getTasks})(TaskList);